@@ -1,12 +1,23 @@
-/* GDCHART 0.10.0dev  GDC_PIE.C  2 Nov 2000 */
-/* Copyright Bruce Verderaime 1998, 1999, 2000 */
+/* GDCHART 0.11.3dev  GDC_PIE.C  11 Mar 2003 */
+/* Copyright Bruce Verderaime 1998-2004 */
 
-#include <stdio.h>
-#include <math.h>
+/* ELLIPSE (angled/perspective view)                                     */
+/* TODO                                                                  */
+/* 	view on/off/%/deg option should be tied to 3d_angle?                 */
+/*		limited in two directions only - by X axis, or Y axis            */
+/*		ELIPSEY [scaling] only one that make sense?                      */
+/*	connections between face&background (effects Y axis only?)           */
+/* 	sizing still doesn't work                                     - DONE */
+/* 	label distances need to be variable about the ellipse/3d_angle       */
+/* #define ELLIPSEX	1.00                                                 */
+/* #define ELLIPSEY	0.30                                                 */
 
 #define GDC_INCL
 #define GDC_LIB
-#include "gdc.h"
+#include "gdc.h"	/* gdc.h before system includes to pick up features */
+
+#include <stdio.h>
+
 #include "gdcpie.h"
 
 /* rem circle:  x = rcos(@), y = rsin(@)	*/
@@ -18,8 +29,8 @@ extern struct	GDC_FONT_T	GDC_fontc[];
 										gdp[1].x = gdp[2].x = x2,	\
 										gdp[2].y = gdp[3].y = y2
 
-#define PX( x )				( cx + (int)( ((float)rad)*sin(pscl*(double)(x)) ) )		/* expects a val */
-#define PY( x )				( cy - (int)( ((float)rad)*cos(pscl*(double)(x)) ) )		/* expects a val */
+#define PX( x )				( cx + (int)( ((float)rad*ellipsex)*sin(pscl*(double)(x)) ) )		/* expects a val */
+#define PY( x )				( cy - (int)( ((float)rad*ellipsey)*cos(pscl*(double)(x)) ) )		/* expects a val */
 
 #define CX( i,d )		( cx                +	\
 						  (d? xdepth_3D: 0) +	\
@@ -34,11 +45,11 @@ extern struct	GDC_FONT_T	GDC_fontc[];
  *   and 3D depth:          d (0: do depth,                  *
  *                             1: no depth adjustment)       *
 \* adjusts for explosion                                     */
-#define IX( i,f,d )		( CX(i,d) + (int)( (double)rad * sin((double)(slice_angle[f][i])) ) )
-#define IY( i,f,d )		( CY(i,d) - (int)( (double)rad * cos((double)(slice_angle[f][i])) ) )
+#define IX( i,f,d )		( CX(i,d) + (int)( (double)rad*ellipsex * sin((double)(slice_angle[f][i])) ) )
+#define IY( i,f,d )		( CY(i,d) - (int)( (double)rad*ellipsey * cos((double)(slice_angle[f][i])) ) )
 /* same as above except o is angle */
-#define OX( i,o,d )		( CX(i,d) + (int)( (double)rad * sin((double)(o)) ) )
-#define OY( i,o,d )		( CY(i,d) - (int)( (double)rad * cos((double)(o)) ) )
+#define OX( i,o,d )		( CX(i,d) + (int)( (double)rad*ellipsex * sin((double)(o)) ) )
+#define OY( i,o,d )		( CY(i,d) - (int)( (double)rad*ellipsey * cos((double)(o)) ) )
 
 #define TO_INT_DEG(o)		(int)rint( (double)((o)/(2.0*M_PI)*360.0) )
 #define TO_INT_DEG_FLOOR(o)	(int)floor( (double)((o)/(2.0*M_PI)*360.0) )
@@ -48,14 +59,14 @@ extern struct	GDC_FONT_T	GDC_fontc[];
 #define MOD_2PI(o)			( (o)>=(2.0*M_PI)? ((o)-(2.0*M_PI)): (((o)<0)? ((o)+(2.0*M_PI)): (o)) )
 #define MOD_360(o)			( (o)>=360? (o)-360: (o) )								/* assume !> 720 */ 
 
-struct tmp_slice_t { int	i;					// original index
-					 char	hidden;				// 'behind' top [3D] pie
-					 float	angle;				// radian 
-					 float	slice; };			// radian
-static float				pie_3D_rad;			// user requested 3D angle in radians
+struct tmp_slice_t { int	i;					/* original index */
+					 char	hidden;				/* 'behind' top [3D] pie */
+					 float	angle;				/* radian  */
+					 float	slice; };			/* radian */
+static float				pie_3D_rad;			/* user requested 3D angle in radians */
 
-// degrees (radians) between angle a, and depth angle
-// 1&2, so comparisons can be done.
+/* degrees (radians) between angle a, and depth angle */
+/* 1&2, so comparisons can be done. */
 #define RAD_DIST1( a )		( (dist_foo1=ABS(((a>-.00001&&a<.00001)?0.00001:a)-pie_3D_rad)), ((dist_foo1>M_PI)? ABS(dist_foo1-2.0*M_PI): dist_foo1) )
 #define RAD_DIST2( a )		( (dist_foo2=ABS(((a>-.00001&&a<.00001)?0.00001:a)-pie_3D_rad)), ((dist_foo2>M_PI)? ABS(dist_foo2-2.0*M_PI): dist_foo2) )
 static float				dist_foo1, dist_foo2;
@@ -65,7 +76,9 @@ static float				dist_foo1, dist_foo2;
  * order by angle opposite (180) of depth angle
  * comparing across 0-360 line
 \* ------------------------------------------------------- */
-static int ocmpr( struct tmp_slice_t *a, struct tmp_slice_t *b )
+static int
+ocmpr( struct tmp_slice_t *a,
+	   struct tmp_slice_t *b )
 {
 	if( RAD_DIST1(a->angle) < RAD_DIST2(b->angle) )
 		return 1;
@@ -114,15 +127,17 @@ GDC_out_pie( short			IMGWIDTH,
 				LineColor,
 				PlotColor,
 				EdgeColor,
-				EdgeColorShd,
-				SliceColor[num_points],
-				SliceColorShd[num_points];
+				EdgeColorShd;
+	CREATE_ARRAY1( SliceColor, int, num_points );		/* int SliceColor[num_points] */
+	CREATE_ARRAY1( SliceColorShd, int, num_points );	/* int SliceColorShd[num_points] */
 
-	float		rad = 0.0;					// radius
+	float		rad = 0.0;					/* radius */
+	float		ellipsex = 1.0;
+	float		ellipsey = 1.0 - (float)GDCPIE_perspective/100.0;
 	float		tot_val = 0.0;
 	float		pscl;
-	int			cx,							// affects PX()
-				cy;							// affects PY()
+	int			cx,							/* affects PX() */
+				cy;							/* affects PY() */
 								/* ~ 1% for a size of 100 pixs */
 								/* label sizes will more dictate this */
 	float		min_grphable = ( GDCPIE_other_threshold < 0?
@@ -131,22 +146,40 @@ GDC_out_pie( short			IMGWIDTH,
 	short		num_slices1 = 0,
 				num_slices2 = 0;
 	char		any_too_small = FALSE;
-	char		others[num_points];
-	float		slice_angle[3][num_points];	// must be used with others[]
+	CREATE_ARRAY1( others, char, num_points );			/* char others[num_points] */
+	CREATE_ARRAY2( slice_angle, float, 3, num_points );	/* float slice_angle[3][num_points] */
+														/* must be used with others[] */
 	char		threeD = ( type == GDC_3DPIE );
 
-	int			xdepth_3D      = 0,			// affects PX()
-				ydepth_3D      = 0;			// affects PY()
-	int			do3Dx = 0,					// reserved for macro use
+	int			xdepth_3D      = 0,			/* affects PX() */
+				ydepth_3D      = 0;			/* affects PY() */
+	int			do3Dx = 0,					/* reserved for macro use */
 				do3Dy = 0;
 
-//	GDCPIE_3d_angle = MOD_360(90-GDCPIE_3d_angle+360);
+	CREATE_ARRAY2( pct_lbl, char, num_points, 16 );			/* sizeof or strlen (GDCPIE_percent_fmt)? */
+	CREATE_ARRAY1( pct_ftsz, struct fnt_sz_t, num_points );	/* struct fnt_sz_t lbl_ftsz[num_points] */
+	CREATE_ARRAY1( lbl_ftsz, struct fnt_sz_t, num_points );	/* struct fnt_sz_t lbl_ftsz[num_points] */
+
+
+#ifdef HAVE_LIBFREETYPE
+	char			*gdcpie_title_font  = GDCPIE_title_font;
+	char			*gdcpie_label_font  = GDCPIE_label_font;
+	double			gdcpie_title_ptsize = GDCPIE_title_ptsize;
+	double			gdcpie_label_ptsize = GDCPIE_label_ptsize;
+#else
+	char			*gdcpie_title_font  = NULL;
+	char			*gdcpie_label_font  = NULL;
+	double			gdcpie_title_ptsize = 0.0;
+	double			gdcpie_label_ptsize = 0.0;
+#endif
+
+/*	GDCPIE_3d_angle = MOD_360(90-GDCPIE_3d_angle+360); */
 	pie_3D_rad = TO_RAD( GDCPIE_3d_angle );
 
 	xdepth_3D      = threeD? (int)( cos((double)MOD_2PI(M_PI_2-pie_3D_rad+2.0*M_PI)) * GDCPIE_3d_depth ): 0;
 	ydepth_3D      = threeD? (int)( sin((double)MOD_2PI(M_PI_2-pie_3D_rad+2.0*M_PI)) * GDCPIE_3d_depth ): 0;
-//	xdepth_3D      = threeD? (int)( cos(pie_3D_rad) * GDCPIE_3d_depth ): 0;
-//	ydepth_3D      = threeD? (int)( sin(pie_3D_rad) * GDCPIE_3d_depth ): 0;
+/*	xdepth_3D      = threeD? (int)( cos(pie_3D_rad) * GDCPIE_3d_depth ): 0; */
+/*	ydepth_3D      = threeD? (int)( sin(pie_3D_rad) * GDCPIE_3d_depth ): 0; */
 
 	load_font_conversions();
 
@@ -159,8 +192,10 @@ GDC_out_pie( short			IMGWIDTH,
 	/* ----- determine pie's radius ----- */
 	{
 	int		title_hgt  = GDCPIE_title? 1			/*  title? horizontal text line */
-									   + GDC_fontc[GDCPIE_title_size].h
-										* (int)cnt_nl( GDCPIE_title, (int*)NULL )
+									   + GDCfnt_sz( GDCPIE_title,
+													GDCPIE_title_size,
+													gdcpie_title_font, gdcpie_title_ptsize, 0.0, NULL ).h
+
 									   + 2:
 									   0;
 	float	last = 0.0;
@@ -168,9 +203,9 @@ GDC_out_pie( short			IMGWIDTH,
 	int		cheight,
 			cwidth;
 
-	// maximum: no labels, explosions
-	// gotta start somewhere
-	rad = (float)MIN( IMGWIDTH/2-(1+ABS(xdepth_3D)), IMGHEIGHT/2-(1+ABS(ydepth_3D))-title_hgt );
+	/* maximum: no labels, explosions */
+	/* gotta start somewhere */
+	rad = (float)MIN( (IMGWIDTH/2)/ellipsex-(1+ABS(xdepth_3D)), (IMGHEIGHT/2)/ellipsey-(1+ABS(ydepth_3D))-title_hgt );
 
 	/* ok fix center, i.e., no floating re labels, explosion, etc. */
 	cx = IMGWIDTH/2 /* - xdepth_3D */ ;
@@ -193,86 +228,107 @@ GDC_out_pie( short			IMGWIDTH,
 			slice_angle[0][i] = this/2.0+last;				/* mid-point on full pie */
 			slice_angle[1][i] = last;						/* 1st on full pie */
 			slice_angle[2][i] = this+last;					/* 2nd on full pie */
-			this_sin        = sin( (double)slice_angle[0][i] );
-			this_cos        = cos( (double)slice_angle[0][i] );
+			this_sin        = ellipsex*sin( (double)slice_angle[0][i] );
+			this_cos        = ellipsey*cos( (double)slice_angle[0][i] );
 
 			if( !GDCPIE_missing || !(GDCPIE_missing[i]) )
 				{
-				short	lbl_wdth,
-						lbl_hgt;
+				short	lbl_wdth = 0,
+						lbl_hgt  = 0;
 				float	this_y_explode_limit,
 						this_x_explode_limit;
 
 				/* start slice label height, width     */
 				/*  accounting for PCT placement, font */
+				pct_ftsz[i].h = 0;
+				pct_ftsz[i].w = 0;
+				if( GDCPIE_percent_fmt &&
+					GDCPIE_percent_labels != GDCPIE_PCT_NONE )
+					{
+					sprintf( pct_lbl[i], GDCPIE_percent_fmt, this_pct * 100.0 );
+					pct_ftsz[i] = GDCfnt_sz( pct_lbl[i],
+											 GDCPIE_label_size,
+											 gdcpie_label_font, gdcpie_label_ptsize, 0.0, NULL );
+					lbl_wdth = pct_ftsz[i].w;
+					lbl_hgt  = pct_ftsz[i].h;
+					}
+
 				if( lbl && lbl[i] )
 					{
-					char	foo[ 16 ];						/* XPG2 compatibility */
-					int		pct_len;
-					int		lbl_len = 0;
-					lbl_hgt = ( cnt_nl(lbl[i], &lbl_len) + (GDCPIE_percent_labels == GDCPIE_PCT_ABOVE ||
-															GDCPIE_percent_labels == GDCPIE_PCT_BELOW? 1: 0) )
-							  * (GDC_fontc[GDCPIE_label_size].h+1);
-					sprintf( foo,
-							 GDCPIE_percent_fmt? GDCPIE_percent_fmt: "",
-							 this_pct * 100.0 );
-					pct_len = GDCPIE_percent_labels == GDCPIE_PCT_NONE? 0: strlen(foo);
-					lbl_wdth = ( GDCPIE_percent_labels == GDCPIE_PCT_RIGHT ||
-								 GDCPIE_percent_labels == GDCPIE_PCT_LEFT? lbl_len+1+pct_len:
-																		   MAX(lbl_len,pct_len) )
-							   * GDC_fontc[GDCPIE_label_size].w;
+					lbl_ftsz[i] = GDCfnt_sz( lbl[i],
+											 GDCPIE_label_size,
+											 gdcpie_label_font, gdcpie_label_ptsize, 0.0, NULL );
+
+					if( GDCPIE_percent_labels == GDCPIE_PCT_ABOVE ||
+						GDCPIE_percent_labels == GDCPIE_PCT_BELOW )
+						{
+						lbl_wdth = MAX( pct_ftsz[i].w, lbl_ftsz[i].w );
+						lbl_hgt  = pct_ftsz[i].h + lbl_ftsz[i].h + 1;
+						}
+					else
+					if( GDCPIE_percent_labels == GDCPIE_PCT_RIGHT ||
+						GDCPIE_percent_labels == GDCPIE_PCT_LEFT )
+						{
+						lbl_wdth = pct_ftsz[i].w + lbl_ftsz[i].w + 1;
+						lbl_hgt  = MAX( pct_ftsz[i].h, lbl_ftsz[i].h );
+						}
+					else /* GDCPIE_PCT_NONE */
+						{
+						lbl_wdth = lbl_ftsz[i].w;
+						lbl_hgt  = lbl_ftsz[i].h;
+						}
 					}
 				else
 					lbl_wdth = lbl_hgt = 0;
 				/* end label height, width */
 				
-				/* diamiter limited by this piont's: explosion, label                 */
+				/* diamiter limited by this point's: explosion, label                 */
 				/* (radius to box @ slice_angle) - (explode) - (projected label size) */
 				/* radius constraint due to labels */
-				this_y_explode_limit = (float)this_cos==0.0? MAXFLOAT:
+				this_y_explode_limit = (float)this_cos==0.0? FLT_MAX:
 										(	(float)( (double)cheight/ABS(this_cos) ) - 
-											(float)( this_explode + (lbl[i]? GDCPIE_label_dist: 0) ) -
+											(float)( this_explode + (lbl&&lbl[i]? GDCPIE_label_dist: 0) ) -
 											(float)( lbl_hgt/2 ) / (float)ABS(this_cos)	);
-				this_x_explode_limit = (float)this_sin==0.0? MAXFLOAT:
+				this_x_explode_limit = (float)this_sin==0.0? FLT_MAX:
 										(	(float)( (double)cwidth/ABS(this_sin) ) - 
-											(float)( this_explode + (lbl[i]? GDCPIE_label_dist: 0) ) -
+											(float)( this_explode + (lbl&&lbl[i]? GDCPIE_label_dist: 0) ) -
 											(float)( lbl_wdth ) / (float)ABS(this_sin)	);
 
 				rad = MIN( rad, this_y_explode_limit );
 				rad = MIN( rad, this_x_explode_limit );
 
-				// ok at this radius (which is most likely larger than final)
-				// adjust for inter-label spacing
-//				if( lbl[i] && *lbl[i] )
-//					{
-//					char which_edge = slice_angle[0][i] > M_PI? +1: -1;		// which semi
-//					last_label_yedge = cheight - (int)( (rad +				// top or bottom of label
-//														(float)(this_explode +
-//														(float)GDCPIE_label_dist)) * (float)this_cos ) +
-//											     ( (GDC_fontc[GDCPIE_label_size].h+1)/2 +
-//													GDC_label_spacing )*which_edge;
-//					}
+				/* ok at this radius (which is most likely larger than final) */
+				/* adjust for inter-label spacing */
+/*				if( lbl[i] && *lbl[i] ) */
+/*					{ */
+/*					char which_edge = slice_angle[0][i] > M_PI? +1: -1;		// which semi */
+/*					last_label_yedge = cheight - (int)( (rad +				// top or bottom of label */
+/*														(float)(this_explode + */
+/*														(float)GDCPIE_label_dist)) * (float)this_cos ) + */
+/*											     ( (GDC_fontc[GDCPIE_label_size].h+1)/2 + */
+/*													GDC_label_spacing )*which_edge; */
+/*					} */
 
 				/* radius constriant due to exploded depth */
 				/* at each edge of the slice, and the middle */
 				/* this is really stupid */
 				/*  this section uses a different algorithm then above, but does the same thing */
 				/*  could be combined, but each is ugly enough! */
-// PROTECT /0
+/* PROTECT /0 */
 				if( threeD )
 					{
 					short	j;
 					int		this_y_explode_pos;
 					int		this_x_explode_pos;
 
-					// first N E S W (actually no need for N)
-					if( (slice_angle[1][i] < M_PI_2 && M_PI_2 < slice_angle[2][i]) &&				// E
+					/* first N E S W (actually no need for N) */
+					if( (slice_angle[1][i] < M_PI_2 && M_PI_2 < slice_angle[2][i]) &&				/* E */
 						(this_x_explode_pos=OX(i,M_PI_2,1)) > cx+cwidth )
 						rad -= (float)ABS( (double)(1+this_x_explode_pos-(cx+cwidth))/sin(M_PI_2) );
-					if( (slice_angle[1][i] < 3.0*M_PI_2 && 3.0*M_PI_2 < slice_angle[2][i]) &&		// W
+					if( (slice_angle[1][i] < 3.0*M_PI_2 && 3.0*M_PI_2 < slice_angle[2][i]) &&		/* W */
 						(this_x_explode_pos=OX(i,3.0*M_PI_2,1)) < cx-cwidth )
 						rad -= (float)ABS( (double)(this_x_explode_pos-(cx+cwidth))/sin(3.0*M_PI_2) );
-					if( (slice_angle[1][i] < M_PI && M_PI < slice_angle[2][i]) &&					// S
+					if( (slice_angle[1][i] < M_PI && M_PI < slice_angle[2][i]) &&					/* S */
 						(this_y_explode_pos=OY(i,M_PI,1)) > cy+cheight )
 						rad -= (float)ABS( (double)(1+this_y_explode_pos-(cy+cheight))/cos(M_PI) );
 
@@ -297,7 +353,7 @@ GDC_out_pie( short			IMGWIDTH,
 		else
 			{
 			others[i] = TRUE;
-			slice_angle[0][i] = -MAXFLOAT;
+			slice_angle[0][i] = -FLT_MAX;
 			}
 		last += this;
 		}
@@ -341,50 +397,50 @@ GDC_out_pie( short			IMGWIDTH,
 	/*              determined by number of labels that'll fit (height) */
 	/* scale to user values */
 	/* ( M_PI / (IMGHEIGHT / (SFONTHGT+1)) ) */
-//	min_grphable = tot_val /
-//				   ( 2.0 * (float)IMGHEIGHT / (float)(SFONTHGT+1+TFONTHGT+2) );
+/*	min_grphable = tot_val / */
+/*				   ( 2.0 * (float)IMGHEIGHT / (float)(SFONTHGT+1+TFONTHGT+2) ); */
 
 
 	if( threeD )
 		{
 		/* draw background shaded pie */
 		{
-		float	rad1 = rad;
+		float	rad1 = rad * 3.0/4.0;
 		for( i=0; i<num_points; ++i )
 			if( !(others[i]) &&
 				(!GDCPIE_missing || !GDCPIE_missing[i]) )
 				{
-				float	rad = rad1;
-
-				gdImageLine( im, CX(i,1), CY(i,1), IX(i,1,1), IY(i,1,1), SliceColorShd[i] );
-				gdImageLine( im, CX(i,1), CY(i,1), IX(i,2,1), IY(i,2,1), SliceColorShd[i] );
+				int		edge_color = GDCPIE_EdgeColor == GDC_NOCOLOR? SliceColorShd[i]:
+				                                                      EdgeColorShd;
 
+				gdImageLine( im, CX(i,1), CY(i,1), IX(i,1,1), IY(i,1,1), edge_color );
+				gdImageLine( im, CX(i,1), CY(i,1), IX(i,2,1), IY(i,2,1), edge_color );
 				gdImageArc( im, CX(i,1), CY(i,1),
-								rad*2, rad*2,
+								(int)(rad*ellipsex*2.0), (int)(rad*ellipsey*2.0),
 								TO_INT_DEG_FLOOR(slice_angle[1][i])+270,
 								TO_INT_DEG_CEIL(slice_angle[2][i])+270,
-								SliceColorShd[i] );
-				rad1 = rad;
-				rad *= 3.0/4.0;
-				gdImageFillToBorder( im, IX(i,0,1), IY(i,0,1), SliceColorShd[i], SliceColorShd[i] );
-				rad = rad1;
-				if( GDCPIE_EdgeColor != GDC_NOCOLOR )
+								edge_color );
+					
+/*				gdImageFilledArc( im, CX(i,1), CY(i,1), */
+/*									  rad*ellipsex*2, rad*ellipsey*2, */
+/*									  TO_INT_DEG_FLOOR(slice_angle[1][i])+270, */
+/*									  TO_INT_DEG_CEIL(slice_angle[2][i])+270, */
+/*									  SliceColorShd[i], */
+/*									  gdPie ); */
+				/* attempt to fill, if slice is wide enough */
+				if( (ABS(IX(i,1,1)-IX(i,2,1)) + ABS(IY(i,1,1)-IY(i,2,1))) > 2 )
 					{
-					gdImageLine( im, CX(i,1), CY(i,1), IX(i,1,1), IY(i,1,1), EdgeColorShd );
-					gdImageLine( im, CX(i,1), CY(i,1), IX(i,2,1), IY(i,2,1), EdgeColorShd );
-					gdImageArc( im, CX(i,1), CY(i,1), 
-									rad*2, rad*2,
-									TO_INT_DEG(slice_angle[1][i])+270, TO_INT_DEG(slice_angle[2][i])+270,
-									EdgeColorShd);
+					float	rad = rad1;										/* local override */
+					gdImageFillToBorder( im, IX(i,0,1), IY(i,0,1), edge_color, SliceColorShd[i] );
 					}
 				}
 		}
 		/* fill in connection to foreground pie */
 		/* this is where we earn our keep */
 		{
-		struct tmp_slice_t	tmp_slice[2*num_points+2];
 		int					t,
 							num_slice_angles = 0;
+		CREATE_ARRAY1( tmp_slice, struct tmp_slice_t, 4*num_points+4 );		/* should only need 2*num_points+2 */
 
 		for( i=0; i<num_points; ++i )
 			if( !GDCPIE_missing || !GDCPIE_missing[i] )
@@ -403,7 +459,7 @@ GDC_out_pie( short			IMGWIDTH,
 				tmp_slice[num_slice_angles].i       = i;
 				tmp_slice[num_slice_angles].slice   = slice_angle[0][i];
 				tmp_slice[num_slice_angles++].angle = slice_angle[2][i];
-				// identify which 2 slices (i) have a tangent parallel to depth angle 
+				/* identify which 2 slices (i) have a tangent parallel to depth angle  */
 				if( slice_angle[1][i]<MOD_2PI(pie_3D_rad+M_PI_2) && slice_angle[2][i]>MOD_2PI(pie_3D_rad+M_PI_2) )
 					{
 					tmp_slice[num_slice_angles].i       = i;
@@ -454,64 +510,74 @@ GDC_out_pie( short			IMGWIDTH,
 				gdImageLine( im, CX(i,0), CY(i,0), CX(i,1), CY(i,1), EdgeColorShd );
 				gdImageLine( im, OX(i,tmp_slice[t].angle,0), OY(i,tmp_slice[t].angle,0),
 								 OX(i,tmp_slice[t].angle,1), OY(i,tmp_slice[t].angle,1),
-							 EdgeColorShd );
+								 EdgeColorShd );
 				}
 			}
+		FREE_ARRAY1( tmp_slice );
 		}
 		}
 
 
 	/* ----- pie face ----- */
 	{
-	// float	last = 0.0;
-	float	rad1 = rad;
+	/* float	last = 0.0; */
+	float	rad1 = rad * 3.0/4.0;
 	for( i=0; i<num_points; ++i )
 		if( !others[i] &&
 			(!GDCPIE_missing || !GDCPIE_missing[i]) )
 			{
-			float	rad = rad1;
-
-			// last += val[i];
-			// EXPLODE_CX_CY( slice_angle[0][i], i );
-			gdImageLine( im, CX(i,0), CY(i,0), IX(i,1,0), IY(i,1,0), SliceColor[i] );
-			gdImageLine( im, CX(i,0), CY(i,0), IX(i,2,0), IY(i,2,0), SliceColor[i] );
-
+			int		edge_color = GDCPIE_EdgeColor == GDC_NOCOLOR? SliceColor[i]:
+																  EdgeColorShd;
+			/* last += val[i]; */
+			/* EXPLODE_CX_CY( slice_angle[0][i], i ); */
+			gdImageLine( im, CX(i,0), CY(i,0), IX(i,1,0), IY(i,1,0), edge_color );
+			gdImageLine( im, CX(i,0), CY(i,0), IX(i,2,0), IY(i,2,0), edge_color );
 			gdImageArc( im, CX(i,0), CY(i,0), 
-							(int)rad*2, (int)rad*2,
-							TO_INT_DEG_FLOOR(slice_angle[1][i])+270,
-							TO_INT_DEG_CEIL(slice_angle[2][i])+270,
-							SliceColor[i] );
-			rad1 = rad;
-			rad *= 3.0/4.0;
-			gdImageFillToBorder( im, IX(i,0,0), IY(i,0,0), SliceColor[i], SliceColor[i] );
+							(int)(rad*ellipsex*2.0), (int)(rad*ellipsey*2.0),
+							(TO_INT_DEG_FLOOR(slice_angle[1][i])+270)%360,
+							(TO_INT_DEG_CEIL(slice_angle[2][i])+270)%360,
+							edge_color );
+			/* antialiasing here */
+			/* likely only on the face? */
+/* bugs in gd2.0.0 */
+/*	arc doesn't honor deg>360 */
+/*	arcs from gdImageFilledArc() don't match with gdImageArc() */
+/*	angles are off */
+/*	doesn't always fill completely */
+/*			gdImageFilledArc( im, CX(i,0), CY(i,0),  */
+/*								  (int)(rad*ellipsex*2.0), (int)(rad*ellipsey*2.0), */
+/*								  (TO_INT_DEG_FLOOR(slice_angle[1][i])+270)%360, */
+/*								  (TO_INT_DEG_CEIL(slice_angle[2][i])+270)%360, */
+/*								  SliceColor[i], */
+/*								  gdPie ); */
+			/* attempt to fill, if slice is wide enough */
+			{
+			float	rad = rad1;										/* local override */
+			if( (ABS(IX(i,1,1)-IX(i,2,1)) + ABS(IY(i,1,1)-IY(i,2,1))) > 2 )
+				{
+				gdImageFillToBorder( im, IX(i,0,0), IY(i,0,0), edge_color, SliceColor[i] );
+				}
 			/* catch missed pixels on narrow slices */
 			gdImageLine( im, CX(i,0), CY(i,0), IX(i,0,0), IY(i,0,0), SliceColor[i] );
-			rad = rad1;
-			if( GDCPIE_EdgeColor != GDC_NOCOLOR )
-				{
-				gdImageLine( im, CX(i,0), CY(i,0), IX(i,1,0), IY(i,1,0), EdgeColor );
-				gdImageLine( im, CX(i,0), CY(i,0), IX(i,2,0), IY(i,2,0), EdgeColor );
-
-				gdImageArc( im, CX(i,0), CY(i,0), 
-								rad*2, rad*2,
-								TO_INT_DEG(slice_angle[1][i])+270, TO_INT_DEG(slice_angle[2][i])+270,
-								EdgeColor );
-				}
+			}
 			}
 	}
 
 	if( GDCPIE_title )
 		{
-		int	title_len;
-
-		cnt_nl( GDCPIE_title, &title_len );
+		struct fnt_sz_t	tftsz      = GDCfnt_sz( GDCPIE_title,
+												GDCPIE_title_size,
+												gdcpie_title_font, gdcpie_title_ptsize, 0.0, NULL );
 		GDCImageStringNL( im,
 						  &GDC_fontc[GDCPIE_title_size],
-						  (IMGWIDTH-title_len*GDC_fontc[GDCPIE_title_size].w)/2,
+						  gdcpie_title_font, gdcpie_title_ptsize,
+						  0.0,
+						  IMGWIDTH/2 - tftsz.w/2,
 						  1,
 						  GDCPIE_title,
 						  LineColor,
-						  GDC_JUSTIFY_CENTER );
+						  GDC_JUSTIFY_CENTER,
+						  NULL );
 		}
 
 	/* labels */
@@ -525,41 +591,16 @@ GDC_out_pie( short			IMGWIDTH,
 			if( !others[i] &&
 				(!GDCPIE_missing || !GDCPIE_missing[i]) )
 				{
-				char	pct_str[ 16 ];
-				int		pct_wdth;
-				int		lbl_wdth;
-				short	num_nl = cnt_nl( lbl[i], &lbl_wdth );
 				int		lblx,  pctx,
 						lbly,  pcty,
 						linex, liney;
 
-				lbl_wdth *= GDC_fontc[GDCPIE_label_size].w;
-				sprintf( pct_str,
-						 GDCPIE_percent_fmt? GDCPIE_percent_fmt: "",
-						 (val[i]/tot_val) * 100.0 );
-				/* in the case of left or right % labels, append or prepend a space */
-				if( GDCPIE_percent_labels == GDCPIE_PCT_RIGHT )
-					{
-					char	foo[16];
-					strcpy( foo, pct_str );
-					sprintf( pct_str, " %s", foo );
-					}
-				else
-				if( GDCPIE_percent_labels == GDCPIE_PCT_LEFT )
-					{
-					pct_str[strlen(pct_str)]   = ' ';
-					pct_str[strlen(pct_str)+1] = '\0';
-					}
-				pct_wdth = GDCPIE_percent_labels == GDCPIE_PCT_NONE?
-							0:
-							strlen(pct_str) * GDC_fontc[GDCPIE_label_size].w;
-
-				lbly = (liney = IY(i,0,0))-( num_nl * (1+GDC_fontc[GDCPIE_label_size].h) ) / 2;
+				lbly = (liney = IY(i,0,0))-lbl_ftsz[i].h / 2;
 				lblx = pctx = linex = IX(i,0,0);
 
 				if( slice_angle[0][i] > M_PI )								/* which semicircle */
 					{
-					lblx -= lbl_wdth;
+					lblx -= lbl_ftsz[i].w;
 					pctx = lblx;
 					++linex;
 					}
@@ -569,44 +610,51 @@ GDC_out_pie( short			IMGWIDTH,
 				switch( GDCPIE_percent_labels )
 					{
 					case GDCPIE_PCT_LEFT:	if( slice_angle[0][i] > M_PI )
-												pctx -= lbl_wdth-1;
+												pctx -= lbl_ftsz[i].w-1;
 											else
-												lblx += pct_wdth+1;
-											pcty = IY(i,0,0) - ( 1+GDC_fontc[GDCPIE_label_size].h ) / 2;
+												lblx += pct_ftsz[i].w+1;
+											pcty = IY(i,0,0) - ( 1+pct_ftsz[i].h ) / 2;
 											break;
 					case GDCPIE_PCT_RIGHT:	if( slice_angle[0][i] > M_PI )
-												lblx -= pct_wdth-1;
+												lblx -= pct_ftsz[i].w-1;
 											else
-												pctx += lbl_wdth+1;
-											pcty = IY(i,0,0) - ( 1+GDC_fontc[GDCPIE_label_size].h ) / 2;
+												pctx += lbl_ftsz[i].w+1;
+											pcty = IY(i,0,0) - ( 1+pct_ftsz[i].h ) / 2;
 											break;
-					case GDCPIE_PCT_ABOVE:	lbly += (1+GDC_fontc[GDCPIE_label_size].h) / 2;
-											pcty = lbly - (GDC_fontc[GDCPIE_label_size].h);
+					case GDCPIE_PCT_ABOVE:	lbly += (1+pct_ftsz[i].h) / 2;
+											pcty = lbly - pct_ftsz[i].h;
 											break;
-					case GDCPIE_PCT_BELOW:	lbly -= (1+GDC_fontc[GDCPIE_label_size].h) / 2;
-											pcty = lbly + (GDC_fontc[GDCPIE_label_size].h) * num_nl;
+					case GDCPIE_PCT_BELOW:	lbly -= (1+pct_ftsz[i].h) / 2;
+											pcty = lbly + lbl_ftsz[i].h;
 											break;
 					case GDCPIE_PCT_NONE:
-					default:
+					default:;
 					}
 
 				if( GDCPIE_percent_labels != GDCPIE_PCT_NONE )
-					gdImageString( im,
-								   GDC_fontc[GDCPIE_label_size].f,
-								   slice_angle[0][i] <= M_PI? pctx:
-															  pctx+lbl_wdth-pct_wdth,
-								   pcty,
-								   pct_str,
-								   LineColor );
+					GDCImageStringNL( im,
+									  &GDC_fontc[GDCPIE_label_size],
+									  gdcpie_label_font, gdcpie_label_ptsize,
+									  0.0,
+									  slice_angle[0][i] <= M_PI? pctx:
+																 pctx+lbl_ftsz[i].w-pct_ftsz[i].w,
+									  pcty,
+									  pct_lbl[i],
+									  LineColor,
+									  GDC_JUSTIFY_CENTER,
+									  NULL );
 				if( lbl[i] )
 					GDCImageStringNL( im,
 									  &GDC_fontc[GDCPIE_label_size],
+									  gdcpie_label_font, gdcpie_label_ptsize,
+									  0.0,
 									  lblx,
 									  lbly,
 									  lbl[i],
 									  LineColor,
 									  slice_angle[0][i] <= M_PI? GDC_JUSTIFY_LEFT:
-																 GDC_JUSTIFY_RIGHT );
+																 GDC_JUSTIFY_RIGHT,
+									  NULL );
 				if( GDCPIE_label_line )
 					{
 					float	rad = liner;
@@ -620,15 +668,24 @@ GDC_out_pie( short			IMGWIDTH,
 		fflush( img_fptr );
 		switch( GDC_image_type )
 			{
-			case GDC_PNG:	gdImagePng( im, img_fptr );						break;
 #ifdef HAVE_JPEG
 			case GDC_JPEG:	gdImageJpeg( im, img_fptr, GDC_jpeg_quality );	break;
 #endif
 			case GDC_WBMP:	gdImageWBMP( im, PlotColor, img_fptr );			break;
-			case GDC_GIF:
-			default:		gdImageGif( im, img_fptr);
+			case GDC_GIF:	gdImageGif( im, img_fptr);						break;
+			case GDC_PNG:
+			default:		gdImagePng( im, img_fptr );
 			}
 
+	FREE_ARRAY1( lbl_ftsz );
+	FREE_ARRAY1( pct_ftsz );
+	FREE_ARRAY2( pct_lbl );
+
+	FREE_ARRAY2( slice_angle );
+	FREE_ARRAY1( others );
+
+	FREE_ARRAY1( SliceColorShd );
+	FREE_ARRAY1( SliceColor );
 	gdImageDestroy(im);
 	return;
 }
